"""runs flask and executes Query.qury() on /api/second_string route"""
import nltk #to filter output text limiting it to 3 lines
from flask import Flask, jsonify, request #standard flask
from flask_ngrok import run_with_ngrok #add token tho or else weird e
from flask_cors import CORS #had cors error fixed with this

from pydantic import BaseModel
from typing import Any



import locale
locale.getpreferredencoding = lambda: "UTF-8"  #dunno why but flask wont install without this

nltk.download('punkt')  # Download the Punkt tokenizer for post processing(is it a thing?)

app = Flask(__name__)
CORS(app)
run_with_ngrok(app)

#Working on changing urls through api(pretty simple tho)

class API(BaseModel):
  """"""
  qury_object: Any
  
# @app.route('/api/first_string', methods=['POST', 'OPTIONS'])
# def process_first_string():
#     if request.method == 'OPTIONS':
#         # Handle the OPTIONS request
#         response_headers = {
#             'Access-Control-Allow-Origin': '*',
#             'Access-Control-Allow-Methods': 'POST',
#             'Access-Control-Allow-Headers': 'Content-Type'
#         }
#         return '', 200, response_headers
#     ##
#     data = request.get_json()
#     first_string = data['first_string']
#     print(first_string)
#     process_data(first_string)

#     # Process the first string and generate the result
#     result = "sites are loaded !" + urls

#     return jsonify({'first_string': result})

@app.route('/api/query', methods=['POST', 'OPTIONS'])
def process_second_string():
    """Querying and Running  through API.qury()"""
    #CORS
    if request.method == 'OPTIONS':
        # Handle the OPTIONS request
        response_headers = {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Methods': 'POST',
            'Access-Control-Allow-Headers': 'Content-Type'
        }
        return '', 200, response_headers


    data = request.get_json()
    print(data)
    second_string = data['query']
    result = API.qury_object(second_string)
    # Process the second string and generate the result
    result = nltk.sent_tokenize(result)[:1]
    print(result)
    return jsonify({'second_string': result})

def start(obj)->Any:
    API(qury_object=obj)
    app.run()
    del API



"""Initialising huggingface model aswell as langchain"""
from langchain import HuggingFacePipeline
from langchain.chains import RetrievalQA
from pydantic import BaseModel
from typing import Any


class MDL(BaseModel):
    retriever: Any
    model_id: str = "bigscience/bloom-560m"
    task: str = "text-generation"
    model_kwargs: Any = {"temperature": 0, "max_length": 512}
    device: int = -1
    llm: Any = 1

    # def __init__(self, retriever, model_id = "bigscience/bloom-560m", task = "text-generation", model_kwargs = {"temperature": 0, "max_length": 512}, device = 0):
    #     self.retriever = retriever
    #     # downloading model
    #     self.model_id = model_id
    #     self.task = task
    #     self.model_kwargs = model_kwargs
    #     self.device = device
    #     self.llm = HuggingFacePipeline.from_model_id(model_id = self.model_id,task = self.task,model_kwargs = self.model_kwargs,device = self.device,)

    #initialising chains
    def chain(self):
        if self.llm == 1:
            self.llm = HuggingFacePipeline.from_model_id(model_id = self.model_id, task = self.task, model_kwargs = self.model_kwargs, device = self.device)
        qa_chain_instrucEmbed = RetrievalQA.from_chain_type(llm=self.llm,
                                                    chain_type="stuff",
                                                    retriever=self.retriever,
                                                    return_source_documents=True)
        return qa_chain_instrucEmbed                                            





"""Calling langchain to predict aswell as doing some post processing"""
import textwrap
from pydantic import BaseModel
from typing import Any

class Query(BaseModel):
    chain: Any
    
    def wrap_text_preserve_newlines(self, text, width=110):
        # Split the input text into lines based on newline characters
        print(type(text))
        lines = text.split('\n')

        # Wrap each line individually
        wrapped_lines = [textwrap.fill(line, width=width) for line in lines]

        # Join the wrapped lines back together using newline characters
        wrapped_text = '\n'.join(wrapped_lines)

        return wrapped_text

    def process_llm_response(self, llm_response):
        print(self.wrap_text_preserve_newlines(llm_response['result']))
        print('\nSources:')
        for source in llm_response["source_documents"]:
            return source.metadata['source']

    def qury(self, query: str):
        print('-------------------Instructor Embeddings------------------\n')
        llm_response = self.chain(query)
        res = self.process_llm_response(llm_response)
        return res


#(scrape->load->split->[dwnld_instructor_e]->create_vec_graph->save(optional)->[download_model->]initialise_chain->query->)->api
from langchain.document_loaders import UnstructuredURLLoader #to load the source text 
from langchain.text_splitter import CharacterTextSplitter #chunking the load data

import faiss#vector graph knowledge base
from langchain.vectorstores import FAISS

from InstructorEmbedding import INSTRUCTOR# InstructorEmbedding
from langchain.embeddings import HuggingFaceInstructEmbeddings

from pydantic import BaseModel
from typing import Optional, Any


# Python code to illustrate the Modules
class SSV(BaseModel):
        urls: Any = ['https://example.com/','https://example.com/']
        chunk_size: int = 500
        chunk_overlap: int = 100 
        model_name: str = "hkunlp/instructor-xl"
        kwargs: int = 3
        instructor_embeddings: Optional[Any]
        data: Optional[Any]

        # def __init__(self, urls = ['https://it.pccoepune.com/','https://it.pccoepune.com/hod'], chunk_size = 500, chunk_overlap = 100, model_name = "hkunlp/instructor-xl", kwargs = 3):
        #         self.urls = urls
        #         self.chunk_size = chunk_size
        #         self.chunk_overlap = chunk_overlap
        #         self.model_name = model_name
        #         self.kwargs = kwargs
        

        # A normal print function
        def scrap(self):
                try:
                        if self.data:
                                print("data already exists")
       
                        else:
                                print("scraping...")                                 
                                loaders = UnstructuredURLLoader(urls=self.urls)
                                self.data = loaders.load()                        
                except:                            
                        print("scraping...")        
                        loaders = UnstructuredURLLoader(urls=self.urls)
                        self.data = loaders.load()

        def split(self):
                #initialised text splitter
                text_splitter = CharacterTextSplitter(separator=' ',
                                      chunk_size=self.chunk_size,
                                      chunk_overlap=self.chunk_overlap)
                #split the text                              
                docs = text_splitter.split_documents(self.data)                              
                return docs                              

        def vec(self, docs):
                #downloading instructor embedding
                try:
                        if self.instructor_embeddings:
                                print("Embedding model already exists")
                        else:
                                self.instructor_embeddings = HuggingFaceInstructEmbeddings(model_name=self.model_name,
                                                model_kwargs={"device": "cuda"})                        
                except:                            
                        self.instructor_embeddings = HuggingFaceInstructEmbeddings(model_name=self.model_name,
                                                      model_kwargs={"device": "cuda"})
            
                #creating vector graph
                db_instructEmbedd = FAISS.from_documents(docs, self.instructor_embeddings)
                retriever = db_instructEmbedd.as_retriever(search_kwargs={"k": self.kwargs})
                return retriever
        
        def ret(self):
                self.scrap()
                retriever = self.vec(self.split())
                return retriever
#urls, chunk size, overlap, search kwargrs, instructor model


# import scrapsplitvec
# import chain
# import query

# from scrapsplitvec import SSV
# from chain import MDL
# from query import Query


# retriever = SSV()
# qa_chain_instrucEmbed = MDL(retriever = retriever, device = 1)
# Query("hello")


from setuptools import setup, find_packages

setup(
    name='langBOTs',
    version='0.0.31',
    author='Rajat S',
    description='Scrape, Langchain, Deploy !',
    url = 'https://github.com/gapirajat/langBOTs',
    packages=find_packages(),
    install_requires=[
        # List any dependencies your package requires here
        'langchain',
        'tiktoken',
        'sentence_transformers',
        'InstructorEmbedding',
        'faiss-cpu',
        'unstructured',
        'flask',
        # 'pyngrtok',
        'flask-cors',
        'pydantic',
    ],
     classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Scientific/Engineering :: Artificial Intelligence',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
